require 'gitkit_client'
require_dependency "gitkite/application_controller"

module Gitkite
  class GitkitController < ApplicationController
    # Redirect from Gitkit to handle signin callback
    def sign_in_callback
    end

    # Show sign in form 
    def sign_in
      if !request.params['mode']
        redirect_to gitkit_sign_in_url(:params => {:mode=>'select'})
      end
    end

    # This will be called after successfull signin
    def sign_in_success
      g_user = signed_in?

      if g_user
        user = {
          :email => g_user.email,
          :user_id => g_user.user_id,
          :name => g_user.name,
          :photo_url => g_user.photo_url,
          :provider_id => g_user.provider_id
        }

        @user = User.find_by(user_id: user[:user_id])

        if @user.nil?
          @user = User.create user
        else
          @user.update user
        end

        session[:user] = @user.id

        if session[:previous_url]
          previous_url = session[:previous_url]
          session.delete :previous_url
          redirect_to previous_url
        else
          redirect_to main_app.root_url
        end
      else
        @user = false
        session.delete :user

        flash.alert = "Sign in failed. Please try again."
        redirect_to gitkit_sign_in_url
      end
    end

    def sign_out
      reset_session
      cookies.delete :gtoken
      redirect_to main_app.root_url
    end

    def send_email
    end

    private
      def signed_in?        
        gitkit_client = GitkitLib::GitkitClient.new(
                          ENV['gk_client_id'],
                          ENV['gk_service_account_email'],
                          Base64.decode64( ENV['gk_service_account_key'] ),
                          gitkit_sign_in_callback_url,
                          ENV['gk_server_api_key'],
                          ENV['gk_project_id']
                        )

        user = gitkit_client.verify_gitkit_token cookies["gtoken"]

        if user.respond_to? 'user_id'
          gitkit_client.get_user_by_id user.user_id
        else
          false
        end
      end
  end
end